﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Data;

namespace InfragisticsChartPerf
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
            Loaded += new RoutedEventHandler(Window1_Loaded);
        }

        void Window1_Loaded(object sender, RoutedEventArgs e)
        {
            textBoxX.Text = "25";
            textBoxY.Text = "25";

            IntensityChart.Scene.GridArea.RenderingOptions.RenderingMode = Infragistics.Windows.Chart.RenderingMode.Performance;
            UpdateIntensityChart();
            IntensityChart.Legend.Visible = false;

            // Rotate the chart so that it is the correct orientation
            System.Windows.Media.Media3D.QuaternionRotation3D oQR3D = new System.Windows.Media.Media3D.QuaternionRotation3D();
            double dZRadians = 90 * (Math.PI / 180);
            double dXRadians = 45 * (Math.PI / 180);
            double dYRadians = 20 * (Math.PI / 180);
            oQR3D.Quaternion = new System.Windows.Media.Media3D.Quaternion(Math.Sin(dXRadians / 2), Math.Sin(dYRadians / 2), Math.Sin(dZRadians / 2), Math.Cos(dZRadians / 2));
            System.Windows.Media.Media3D.RotateTransform3D oRT3D = new System.Windows.Media.Media3D.RotateTransform3D(oQR3D);
            IntensityChart.Transform3D = oRT3D;

            // Wire event handlers for rotation
            IntensityChart.MouseDown += new MouseButtonEventHandler(IntensityChart_MouseDown);
            IntensityChart.MouseMove += new MouseEventHandler(IntensityChart_MouseMove);
            IntensityChart.MouseUp += new MouseButtonEventHandler(IntensityChart_MouseUp);
        }

        ///******************************************************************************
        /// <Author>Rob Karpelowitz</Author>
        /// <Date>03/14/2010</Date>
        /// <Type>Event Handler</Type>
        /// <summary>
        ///     Mouse up handler.  Terminates mouse drag activity.
        /// </summary>
        /// <remarks>
        /// </remarks>
        ///******************************************************************************
        private double m_dPreviousMouseX = 0;
        private double m_dPreviousMouseY = 0;
        private bool m_bMouseDown = false;
        void IntensityChart_MouseUp(object sender, MouseButtonEventArgs e)
        {
            m_bMouseDown = false;
        }

        ///******************************************************************************
        /// <Author>Rob Karpelowitz</Author>
        /// <Date>03/14/2010</Date>
        /// <Type>Event Handler</Type>
        /// <summary>
        ///     Mouse move handler.  Changes chart rotation.
        /// </summary>
        /// <remarks>
        /// </remarks>
        ///******************************************************************************
        void IntensityChart_MouseMove(object sender, MouseEventArgs e)
        {
            if (m_bMouseDown)
            {
                // Translate mouse movement to x,y rotation
                double dDeltaX = (e.GetPosition(this).X - m_dPreviousMouseX);
                double dDeltaY = (e.GetPosition(this).Y - m_dPreviousMouseY);

                int iXDir = (dDeltaX != 0) ? (dDeltaX < 0) ? -1 : 1 : 0;
                int iYDir = (dDeltaY != 0) ? (dDeltaY < 0) ? 1 : -1 : 0;

                double dXRadians = iXDir * 2 * (Math.PI / 180);
                double dYRadians = iYDir * 2 * (Math.PI / 180);

                // Limit the rotation to keep the chart in view
                System.Windows.Media.Media3D.RotateTransform3D oRT3D = (System.Windows.Media.Media3D.RotateTransform3D)IntensityChart.Transform3D;
                System.Windows.Media.Media3D.Quaternion oQ = ((System.Windows.Media.Media3D.QuaternionRotation3D)oRT3D.Rotation).Quaternion;
                if (Math.Asin(oQ.X) + dXRadians < 0) dXRadians = 0;
                if (Math.Asin(oQ.Y) + dYRadians < 0) dYRadians = 0;
                if (Math.Asin(oQ.X) + dXRadians > 45 * (Math.PI / 180)) dXRadians = 0;
                if (Math.Asin(oQ.Y) + dYRadians > .34) dYRadians = 0;

                // Build the new rotation and apply
                System.Windows.Media.Media3D.Quaternion oNewQ = new System.Windows.Media.Media3D.Quaternion(Math.Sin(Math.Asin(oQ.X) + dXRadians), Math.Sin(Math.Asin(oQ.Y) + dYRadians), oQ.Z, oQ.W);
                System.Windows.Media.Media3D.QuaternionRotation3D oQR3D = new System.Windows.Media.Media3D.QuaternionRotation3D();
                oQR3D.Quaternion = oNewQ;
                IntensityChart.Transform3D = new System.Windows.Media.Media3D.RotateTransform3D(oQR3D);
            }
        }

        ///******************************************************************************
        /// <Author>Rob Karpelowitz</Author>
        /// <Date>03/14/2010</Date>
        /// <Type>Event Handler</Type>
        /// <summary>
        ///     Mouse down handler.  Initiates mouse drag activity.
        /// </summary>
        /// <remarks>
        /// </remarks>
        ///******************************************************************************
        void IntensityChart_MouseDown(object sender, MouseButtonEventArgs e)
        {
            m_dPreviousMouseX = e.GetPosition(this).X;
            m_dPreviousMouseY = e.GetPosition(this).Y;
            m_bMouseDown = true;
        }


        ///******************************************************************************
        /// <Author>Rob Karpelowitz</Author>
        /// <Date>03/14/2010</Date>
        /// <Type>Method</Type>
        /// <summary>
        ///     Updates the chart when the region changes (e.g. called when the region
        ///     changes by the datainspector tool).
        /// </summary>
        /// <remarks>
        /// </remarks>
        ///******************************************************************************
        public void UpdateIntensityChart()
        {
            DataTable oDT = BuildChartData();

            // Bind the data to the existing series
            // Build the series
            IntensityChart.RefreshEnabled = false;
            IntensityChart.Series.Clear();
            foreach (DataColumn oDC in oDT.Columns)
            {
                Infragistics.Windows.Chart.Series oS = new Infragistics.Windows.Chart.Series();
                oS.DataMapping = oDC.ColumnName;
                oS.Label = oDC.ColumnName;
                oS.DataSource = oDT;
                oS.ChartType = Infragistics.Windows.Chart.ChartType.Bar;
                IntensityChart.Series.Add(oS);
            }

            IntensityChart.RefreshEnabled = true;
        }

        private DataTable BuildChartData()
        {
            // Create a datatable and add columns to it
            DataTable oDT = new DataTable();
            for (double iCol = 0; iCol < Convert.ToDouble(textBoxX.Text); iCol++)
            {
                string strColName = string.Format("X{0}", iCol);
                oDT.Columns.Add(new DataColumn(strColName, typeof(float)));
            }

            // Add rows (Y coord), with the intensity values

            for (int iRow = 0; iRow < Convert.ToInt32(textBoxY.Text); iRow++)
            {
                DataRow oDRow = oDT.NewRow();

                for (int iJIdx = 0; iJIdx < Convert.ToInt32(textBoxX.Text); iJIdx++)
                {
                    oDRow[iJIdx] = 120;
                }

                oDT.Rows.Add(oDRow);
            }

            return oDT;
        }

        private void button1_Click(object sender, RoutedEventArgs e)
        {
            UpdateIntensityChart();
        }
    }
}
